@@ -16,35 +16,13 @@
 //!
 //! ```
 //! use std::io::{Cursor, Read};
-//! use bitstream_io::{BigEndian, BitReader, BitRead, ByteReader, ByteRead, FromBitStream, FromByteStream, LittleEndian};
-//!
-//! let flac: Vec<u8> = vec![0x66,0x4c,0x61,0x43,0x00,0x00,0x00,0x22,
-//!                          0x10,0x00,0x10,0x00,0x00,0x06,0x06,0x00,
-//!                          0x21,0x62,0x0a,0xc4,0x42,0xf0,0x00,0x04,
-//!                          0xa6,0xcc,0xfa,0xf2,0x69,0x2f,0xfd,0xec,
-//!                          0x2d,0x5b,0x30,0x01,0x76,0xb4,0x62,0x88,
-//!                          0x7d,0x92,0x04,0x00,0x00,0x7a,0x20,0x00,
-//!                          0x00,0x00,0x72,0x65,0x66,0x65,0x72,0x65,
-//!                          0x6e,0x63,0x65,0x20,0x6c,0x69,0x62,0x46,
-//!                          0x4c,0x41,0x43,0x20,0x31,0x2e,0x31,0x2e,
-//!                          0x34,0x20,0x32,0x30,0x30,0x37,0x30,0x32,
-//!                          0x31,0x33,0x04,0x00,0x00,0x00,0x16,0x00,
-//!                          0x00,0x00,0x74,0x69,0x74,0x6c,0x65,0x3d,
-//!                          0x32,0x63,0x68,0x20,0x34,0x34,0x31,0x30,
-//!                          0x30,0x20,0x20,0x31,0x36,0x62,0x69,0x74,
-//!                          0x10,0x00,0x00,0x00,0x61,0x6c,0x62,0x75,
-//!                          0x6d,0x3d,0x54,0x65,0x73,0x74,0x20,0x41,
-//!                          0x6c,0x62,0x75,0x6d,0x0f,0x00,0x00,0x00,
-//!                          0x61,0x72,0x74,0x69,0x73,0x74,0x3d,0x41,
-//!                          0x73,0x73,0x6f,0x72,0x74,0x65,0x64,0x0d,
-//!                          0x00,0x00,0x00,0x74,0x72,0x61,0x63,0x6b,
-//!                          0x6e,0x75,0x6d,0x62,0x65,0x72,0x3d,0x31];
+//! use bitstream_io::*;
 //!
 //! #[derive(Debug, PartialEq, Eq)]
 //! struct BlockHeader {
-//!     last_block: bool,
-//!     block_type: u8,
-//!     block_size: u32,
+//!     last_block: bool,  // 1 bit
+//!     block_type: u8,    // 7 bits
+//!     block_size: u32,   // 24 bits
 //! }
 //!
 //! impl FromBitStream for BlockHeader {
@@ -53,23 +31,23 @@
 //!     fn from_reader<R: BitRead + ?Sized>(r: &mut R) -> std::io::Result<Self> {
 //!         Ok(Self {
 //!             last_block: r.read_bit()?,
-//!             block_type: r.read(7)?,
-//!             block_size: r.read(24)?,
+//!             block_type: r.read_in::<7, _>()?,
+//!             block_size: r.read_in::<24, _>()?,
 //!         })
 //!     }
 //! }
 //!
 //! #[derive(Debug, PartialEq, Eq)]
 //! struct Streaminfo {
-//!     minimum_block_size: u16,
-//!     maximum_block_size: u16,
-//!     minimum_frame_size: u32,
-//!     maximum_frame_size: u32,
-//!     sample_rate: u32,
-//!     channels: u8,
-//!     bits_per_sample: u8,
-//!     total_samples: u64,
-//!     md5: [u8; 16],
+//!     minimum_block_size: u16,  // 16 bits
+//!     maximum_block_size: u16,  // 16 bits
+//!     minimum_frame_size: u32,  // 24 bits
+//!     maximum_frame_size: u32,  // 24 bits
+//!     sample_rate: u32,         // 20 bits
+//!     channels: u8,             // 3 bits
+//!     bits_per_sample: u8,      // 5 bits
+//!     total_samples: u64,       // 36 bits
+//!     md5: [u8; 16],            // 16 bytes
 //! }
 //!
 //! impl FromBitStream for Streaminfo {
@@ -79,13 +57,13 @@
 //!         Ok(Self {
 //!             minimum_block_size: r.read_to()?,
 //!             maximum_block_size: r.read_to()?,
-//!             minimum_frame_size: r.read(24)?,
-//!             maximum_frame_size: r.read(24)?,
-//!             sample_rate: r.read(20)?,
-//!             channels: r.read::<u8>(3)? + 1,
-//!             bits_per_sample: r.read::<u8>(5)? + 1,
-//!             total_samples: r.read(36)?,
-//!             md5: r.read_to()?,
+//!             minimum_frame_size: r.read_in::<24, _>()?,
+//!             maximum_frame_size: r.read_in::<24, _>()?,
+//!             sample_rate:        r.read_in::<20, _>()?,
+//!             channels:           r.read_in::<3, u8>()? + 1,
+//!             bits_per_sample:    r.read_in::<5, u8>()? + 1,
+//!             total_samples:      r.read_in::<36, _>()?,
+//!             md5:                r.read_to()?,
 //!         })
 //!     }
 //! }
@@ -119,6 +97,29 @@
 //!    }
 //! }
 //!
+//! // test FLAC file data
+//! let flac: Vec<u8> = vec![0x66,0x4c,0x61,0x43,0x00,0x00,0x00,0x22,
+//!                          0x10,0x00,0x10,0x00,0x00,0x06,0x06,0x00,
+//!                          0x21,0x62,0x0a,0xc4,0x42,0xf0,0x00,0x04,
+//!                          0xa6,0xcc,0xfa,0xf2,0x69,0x2f,0xfd,0xec,
+//!                          0x2d,0x5b,0x30,0x01,0x76,0xb4,0x62,0x88,
+//!                          0x7d,0x92,0x04,0x00,0x00,0x7a,0x20,0x00,
+//!                          0x00,0x00,0x72,0x65,0x66,0x65,0x72,0x65,
+//!                          0x6e,0x63,0x65,0x20,0x6c,0x69,0x62,0x46,
+//!                          0x4c,0x41,0x43,0x20,0x31,0x2e,0x31,0x2e,
+//!                          0x34,0x20,0x32,0x30,0x30,0x37,0x30,0x32,
+//!                          0x31,0x33,0x04,0x00,0x00,0x00,0x16,0x00,
+//!                          0x00,0x00,0x74,0x69,0x74,0x6c,0x65,0x3d,
+//!                          0x32,0x63,0x68,0x20,0x34,0x34,0x31,0x30,
+//!                          0x30,0x20,0x20,0x31,0x36,0x62,0x69,0x74,
+//!                          0x10,0x00,0x00,0x00,0x61,0x6c,0x62,0x75,
+//!                          0x6d,0x3d,0x54,0x65,0x73,0x74,0x20,0x41,
+//!                          0x6c,0x62,0x75,0x6d,0x0f,0x00,0x00,0x00,
+//!                          0x61,0x72,0x74,0x69,0x73,0x74,0x3d,0x41,
+//!                          0x73,0x73,0x6f,0x72,0x74,0x65,0x64,0x0d,
+//!                          0x00,0x00,0x00,0x74,0x72,0x61,0x63,0x6b,
+//!                          0x6e,0x75,0x6d,0x62,0x65,0x72,0x3d,0x31];
+//!
 //! let mut cursor = Cursor::new(&flac);
 //!
 //! let mut reader = BitReader::endian(&mut cursor, BigEndian);
@@ -170,7 +171,14 @@
 
 #![warn(missing_docs)]
 
-use std::io;
+#[cfg(feature = "alloc")]
+use alloc::vec;
+#[cfg(feature = "alloc")]
+use alloc::vec::Vec;
+#[cfg(feature = "alloc")]
+use core2::io::{self, SeekFrom};
+#[cfg(not(feature = "alloc"))]
+use std::io::{self, SeekFrom};
 
 use super::{
     huffman::ReadHuffmanTree, BitQueue, Endianness, Numeric, PhantomData, Primitive, SignedNumeric,
@@ -199,18 +207,49 @@ pub trait BitRead {
     where
         U: Numeric;
 
+    /// Reads an unsigned value from the stream with
+    /// the given constant number of bits.
+    ///
+    /// # Errors
+    ///
+    /// Passes along any I/O error from the underlying stream.
+    /// A compile-time error occurs if the given number of bits
+    /// is larger than the output type.
+    fn read_in<const BITS: u32, U>(&mut self) -> io::Result<U>
+    where
+        U: Numeric,
+    {
+        self.read(BITS)
+    }
+
     /// Reads a twos-complement signed value from the stream with
     /// the given number of bits.
     ///
     /// # Errors
     ///
     /// Passes along any I/O error from the underlying stream.
+    /// Returns an error if the number of bits is 0,
+    /// since one bit is always needed for the sign.
     /// Also returns an error if the output type is too small
     /// to hold the requested number of bits.
     fn read_signed<S>(&mut self, bits: u32) -> io::Result<S>
     where
         S: SignedNumeric;
 
+    /// Reads a twos-complement signed value from the stream with
+    /// the given constant number of bits.
+    ///
+    /// # Errors
+    ///
+    /// Passes along any I/O error from the underlying stream.
+    /// A compile-time error occurs if the number of bits is 0,
+    /// since one bit is always needed for the sign.
+    /// A compile-time error occurs if the given number of bits
+    /// is larger than the output type.
+    fn read_signed_in<const BITS: u32, S>(&mut self) -> io::Result<S>
+    where
+        S: SignedNumeric;
+
     /// Reads whole value from the stream whose size in bits is equal
     /// to its type's size.
     ///
@@ -257,7 +296,7 @@ pub trait BitRead {
     /// Passes along any I/O error from the underlying stream.
     fn read_bytes(&mut self, buf: &mut [u8]) -> io::Result<()> {
         for b in buf.iter_mut() {
-            *b = self.read(8)?;
+            *b = self.read_in::<8, _>()?;
         }
         Ok(())
     }
@@ -555,6 +594,49 @@ impl<R: io::Read, E: Endianness> BitRead for BitReader<R, E> {
         }
     }
 
+    /// # Examples
+    /// ```
+    /// use std::io::{Read, Cursor};
+    /// use bitstream_io::{BigEndian, BitReader, BitRead};
+    /// let data = [0b10110111];
+    /// let mut reader = BitReader::endian(Cursor::new(&data), BigEndian);
+    /// assert_eq!(reader.read_in::<1, u8>().unwrap(), 0b1);
+    /// assert_eq!(reader.read_in::<2, u8>().unwrap(), 0b01);
+    /// assert_eq!(reader.read_in::<5, u8>().unwrap(), 0b10111);
+    /// ```
+    ///
+    /// ```
+    /// use std::io::{Read, Cursor};
+    /// use bitstream_io::{LittleEndian, BitReader, BitRead};
+    /// let data = [0b10110111];
+    /// let mut reader = BitReader::endian(Cursor::new(&data), LittleEndian);
+    /// assert_eq!(reader.read_in::<1, u8>().unwrap(), 0b1);
+    /// assert_eq!(reader.read_in::<2, u8>().unwrap(), 0b11);
+    /// assert_eq!(reader.read_in::<5, u8>().unwrap(), 0b10110);
+    /// ```
+    #[inline]
+    fn read_in<const BITS: u32, U>(&mut self) -> io::Result<U>
+    where
+        U: Numeric,
+    {
+        const {
+            assert!(BITS <= U::BITS_SIZE, "excessive bits for type read");
+        }
+
+        let bitqueue_len = self.bitqueue.len();
+        if BITS <= bitqueue_len {
+            Ok(U::from_u8(self.bitqueue.pop_fixed::<BITS>()))
+        } else {
+            let mut bits = BITS;
+            let mut acc = BitQueue::from_value(U::from_u8(self.bitqueue.pop_all()), bitqueue_len);
+            bits -= bitqueue_len;
+
+            read_aligned(&mut self.reader, bits / 8, &mut acc)?;
+            read_unaligned(&mut self.reader, bits % 8, &mut acc, &mut self.bitqueue)?;
+            Ok(acc.value())
+        }
+    }
+
     /// # Examples
     /// ```
     /// use std::io::{Read, Cursor};
@@ -589,7 +671,47 @@ impl<R: io::Read, E: Endianness> BitRead for BitReader<R, E> {
     where
         S: SignedNumeric,
     {
-        E::read_signed(self, bits)
+        match bits {
+            0 => Err(io::Error::new(
+                io::ErrorKind::InvalidInput,
+                "signed reads need at least 1 bit for sign",
+            )),
+            bits if bits <= S::BITS_SIZE => E::read_signed(self, bits),
+            _ => Err(io::Error::new(
+                io::ErrorKind::InvalidInput,
+                "excessive bits for type read",
+            )),
+        }
+    }
+
+    /// # Examples
+    /// ```
+    /// use std::io::{Read, Cursor};
+    /// use bitstream_io::{BigEndian, BitReader, BitRead};
+    /// let data = [0b10110111];
+    /// let mut reader = BitReader::endian(Cursor::new(&data), BigEndian);
+    /// assert_eq!(reader.read_signed_in::<4, i8>().unwrap(), -5);
+    /// assert_eq!(reader.read_signed_in::<4, i8>().unwrap(), 7);
+    /// ```
+    ///
+    /// ```
+    /// use std::io::{Read, Cursor};
+    /// use bitstream_io::{LittleEndian, BitReader, BitRead};
+    /// let data = [0b10110111];
+    /// let mut reader = BitReader::endian(Cursor::new(&data), LittleEndian);
+    /// assert_eq!(reader.read_signed_in::<4, i8>().unwrap(), 7);
+    /// assert_eq!(reader.read_signed_in::<4, i8>().unwrap(), -5);
+    /// ```
+    #[inline]
+    fn read_signed_in<const BITS: u32, S>(&mut self) -> io::Result<S>
+    where
+        S: SignedNumeric,
+    {
+        const {
+            assert!(BITS > 0, "signed reads need at least 1 bit for sign");
+            assert!(BITS <= S::BITS_SIZE, "excessive bits for type read");
+        }
+        E::read_signed_fixed::<_, BITS, S>(self)
     }
 
     #[inline]
@@ -628,7 +750,7 @@ impl<R: io::Read, E: Endianness> BitRead for BitReader<R, E> {
     /// assert_eq!(reader.read::<u8>(5).unwrap(), 0b10110);
     /// ```
     fn skip(&mut self, mut bits: u32) -> io::Result<()> {
-        use std::cmp::min;
+        use core::cmp::min;
 
         let to_drop = min(self.bitqueue.len(), bits);
         if to_drop != 0 {
@@ -656,7 +778,7 @@ impl<R: io::Read, E: Endianness> BitRead for BitReader<R, E> {
             self.reader.read_exact(buf)
         } else {
             for b in buf.iter_mut() {
-                *b = self.read(8)?;
+                *b = self.read_in::<8, _>()?;
             }
             Ok(())
         }
@@ -825,7 +947,7 @@ where
     /// ```
     #[inline]
     pub fn position_in_bits(&mut self) -> io::Result<u64> {
-        let bytes = self.reader.stream_position()?;
+        let bytes = self.reader.seek(SeekFrom::Current(0))?;
         Ok(bytes * 8 - (self.bitqueue.len() as u64))
     }
 }
@@ -876,7 +998,7 @@ where
 {
     let mut byte = 0;
     reader
-        .read_exact(std::slice::from_mut(&mut byte))
+        .read_exact(core::slice::from_mut(&mut byte))
         .map(|()| byte)
 }
 
@@ -890,7 +1012,7 @@ where
         let mut buf = N::buffer();
         reader.read_exact(&mut buf.as_mut()[0..bytes as usize])?;
         for b in &buf.as_ref()[0..bytes as usize] {
-            acc.push(8, N::from_u8(*b));
+            acc.push_fixed::<8>(N::from_u8(*b));
         }
     }
     Ok(())
@@ -900,7 +1022,7 @@ fn skip_aligned<R>(mut reader: R, mut bytes: u32) -> io::Result<()>
 where
     R: io::Read,
 {
-    use std::cmp::min;
+    use core::cmp::min;
 
     /*skip up to 8 bytes at a time
     (unlike with read_aligned, "bytes" may be larger than any native type)*/
