@@ -21,9 +21,9 @@
 //!
 //! #[derive(Debug, PartialEq, Eq)]
 //! struct BlockHeader {
-//!     last_block: bool,
-//!     block_type: u8,
-//!     block_size: u32,
+//!     last_block: bool,  // 1 bit
+//!     block_type: u8,    // 7 bits
+//!     block_size: u32,   // 24 bits
 //! }
 //!
 //! impl ToBitStream for BlockHeader {
@@ -31,22 +31,22 @@
 //!
 //!     fn to_writer<W: BitWrite + ?Sized>(&self, w: &mut W) -> std::io::Result<()> {
 //!         w.write_bit(self.last_block)?;
-//!         w.write(7, self.block_type)?;
-//!         w.write(24, self.block_size)
+//!         w.write_out::<7, _>(self.block_type)?;
+//!         w.write_out::<24, _>(self.block_size)
 //!     }
 //! }
 //!
 //! #[derive(Debug, PartialEq, Eq)]
 //! struct Streaminfo {
-//!     minimum_block_size: u16,
-//!     maximum_block_size: u16,
-//!     minimum_frame_size: u32,
-//!     maximum_frame_size: u32,
-//!     sample_rate: u32,
-//!     channels: u8,
-//!     bits_per_sample: u8,
-//!     total_samples: u64,
-//!     md5: [u8; 16],
+//!     minimum_block_size: u16,  // 16 bits
+//!     maximum_block_size: u16,  // 16 bits
+//!     minimum_frame_size: u32,  // 24 bits
+//!     maximum_frame_size: u32,  // 24 bits
+//!     sample_rate: u32,         // 20 bits
+//!     channels: u8,             // 3 bits
+//!     bits_per_sample: u8,      // 5 bits
+//!     total_samples: u64,       // 36 bits
+//!     md5: [u8; 16],            // 16 bytes
 //! }
 //!
 //! impl ToBitStream for Streaminfo {
@@ -55,12 +55,12 @@
 //!     fn to_writer<W: BitWrite + ?Sized>(&self, w: &mut W) -> std::io::Result<()> {
 //!         w.write_from(self.minimum_block_size)?;
 //!         w.write_from(self.maximum_block_size)?;
-//!         w.write(24, self.minimum_frame_size)?;
-//!         w.write(24, self.maximum_frame_size)?;
-//!         w.write(20, self.sample_rate)?;
-//!         w.write(3, self.channels - 1)?;
-//!         w.write(5, self.bits_per_sample - 1)?;
-//!         w.write(36, self.total_samples)?;
+//!         w.write_out::<24, _>(self.minimum_frame_size)?;
+//!         w.write_out::<24, _>(self.maximum_frame_size)?;
+//!         w.write_out::<20, _>(self.sample_rate)?;
+//!         w.write_out::<3,  _>(self.channels - 1)?;
+//!         w.write_out::<5,  _>(self.bits_per_sample - 1)?;
+//!         w.write_out::<36, _>(self.total_samples)?;
 //!         w.write_bytes(&self.md5)
 //!     }
 //! }
@@ -163,9 +163,18 @@
 
 #![warn(missing_docs)]
 
-use std::convert::From;
+#[cfg(feature = "alloc")]
+use alloc::boxed::Box;
+#[cfg(feature = "alloc")]
+use alloc::vec::Vec;
+#[cfg(feature = "alloc")]
+use core2::io;
+
+#[cfg(not(feature = "alloc"))]
 use std::io;
-use std::ops::{AddAssign, Rem};
+
+use core::convert::From;
+use core::ops::{AddAssign, Rem};
 
 use super::{
     huffman::WriteHuffmanTree, BitQueue, Endianness, Numeric, PhantomData, Primitive, SignedNumeric,
@@ -316,6 +325,23 @@ pub trait BitWrite {
     where
         U: Numeric;
 
+    /// Writes an unsigned value to the stream using the given
+    /// const number of bits.
+    ///
+    /// # Errors
+    ///
+    /// Passes along any I/O error from the underlying stream.
+    /// Returns an error if the value is too large
+    /// to fit the given number of bits.
+    /// A compile-time error occurs if the given number of bits
+    /// is larger than the output type.
+    fn write_out<const BITS: u32, U>(&mut self, value: U) -> io::Result<()>
+    where
+        U: Numeric,
+    {
+        self.write(BITS, value)
+    }
+
     /// Writes a twos-complement signed value to the stream
     /// with the given number of bits.
     ///
@@ -324,12 +350,30 @@ pub trait BitWrite {
     /// Passes along any I/O error from the underlying stream.
     /// Returns an error if the input type is too small
     /// to hold the given number of bits.
+    /// Returns an error if the number of bits is 0,
+    /// since one bit is always needed for the sign.
     /// Returns an error if the value is too large
     /// to fit the given number of bits.
     fn write_signed<S>(&mut self, bits: u32, value: S) -> io::Result<()>
     where
         S: SignedNumeric;
 
+    /// Writes a twos-complement signed value to the stream
+    /// with the given const number of bits.
+    ///
+    /// # Errors
+    ///
+    /// Passes along any I/O error from the underlying stream.
+    /// Returns an error if the value is too large
+    /// to fit the given number of bits.
+    /// A compile-time error occurs if the number of bits is 0,
+    /// since one bit is always needed for the sign.
+    /// A compile-time error occurs if the given number of bits
+    /// is larger than the output type.
+    fn write_signed_out<const BITS: u32, S>(&mut self, value: S) -> io::Result<()>
+    where
+        S: SignedNumeric;
+
     /// Writes whole value to the stream whose size in bits
     /// is equal to its type's size.
     ///
@@ -372,7 +416,7 @@ pub trait BitWrite {
     /// ```
     #[inline]
     fn write_bytes(&mut self, buf: &[u8]) -> io::Result<()> {
-        buf.iter().try_for_each(|b| self.write(8, *b))
+        buf.iter().try_for_each(|b| self.write_out::<8, _>(*b))
     }
 
     /// Writes `value` number of 1 bits to the stream
@@ -624,6 +668,61 @@ impl<W: io::Write, E: Endianness> BitWrite for BitWriter<W, E> {
         }
     }
 
+    /// # Examples
+    /// ```
+    /// use std::io::Write;
+    /// use bitstream_io::{BigEndian, BitWriter, BitWrite};
+    /// let mut writer = BitWriter::endian(Vec::new(), BigEndian);
+    /// writer.write_out::<1, _>(0b1).unwrap();
+    /// writer.write_out::<2, _>(0b01).unwrap();
+    /// writer.write_out::<5, _>(0b10111).unwrap();
+    /// assert_eq!(writer.into_writer(), [0b10110111]);
+    /// ```
+    ///
+    /// ```
+    /// use std::io::Write;
+    /// use bitstream_io::{LittleEndian, BitWriter, BitWrite};
+    /// let mut writer = BitWriter::endian(Vec::new(), LittleEndian);
+    /// writer.write_out::<1, _>(0b1).unwrap();
+    /// writer.write_out::<2, _>(0b11).unwrap();
+    /// writer.write_out::<5, _>(0b10110).unwrap();
+    /// assert_eq!(writer.into_writer(), [0b10110111]);
+    /// ```
+    ///
+    /// ```
+    /// use std::io::{Write, sink};
+    /// use bitstream_io::{BigEndian, BitWriter, BitWrite};
+    /// let mut w = BitWriter::endian(sink(), BigEndian);
+    /// assert!(w.write_out::<1, _>(2).is_err());      // can't write   2 in 1 bit
+    /// assert!(w.write_out::<2, _>(4).is_err());      // can't write   4 in 2 bits
+    /// assert!(w.write_out::<3, _>(8).is_err());      // can't write   8 in 3 bits
+    /// assert!(w.write_out::<4, _>(16).is_err());     // can't write  16 in 4 bits
+    /// ```
+    fn write_out<const BITS: u32, U>(&mut self, value: U) -> io::Result<()>
+    where
+        U: Numeric,
+    {
+        const {
+            assert!(BITS <= U::BITS_SIZE, "excessive bits for type written");
+        }
+
+        if (BITS < U::BITS_SIZE) && (value >= (U::ONE << BITS)) {
+            Err(io::Error::new(
+                io::ErrorKind::InvalidInput,
+                "excessive value for bits written",
+            ))
+        } else if BITS < self.bitqueue.remaining_len() {
+            self.bitqueue.push_fixed::<BITS>(value.to_u8());
+            Ok(())
+        } else {
+            let mut acc = BitQueue::from_value(value, BITS);
+            write_unaligned(&mut self.writer, &mut acc, &mut self.bitqueue)?;
+            write_aligned(&mut self.writer, &mut acc)?;
+            self.bitqueue.push(acc.len(), acc.value().to_u8());
+            Ok(())
+        }
+    }
+
     /// # Examples
     /// ```
     /// use std::io::Write;
@@ -647,7 +746,47 @@ impl<W: io::Write, E: Endianness> BitWrite for BitWriter<W, E> {
     where
         S: SignedNumeric,
     {
-        E::write_signed(self, bits, value)
+        match bits {
+            0 => Err(io::Error::new(
+                io::ErrorKind::InvalidInput,
+                "signed writes need at least 1 bit for sign",
+            )),
+            bits if bits <= S::BITS_SIZE => E::write_signed(self, bits, value),
+            _ => Err(io::Error::new(
+                io::ErrorKind::InvalidInput,
+                "excessive bits for type written",
+            )),
+        }
+    }
+
+    /// # Examples
+    /// ```
+    /// use std::io::Write;
+    /// use bitstream_io::{BigEndian, BitWriter, BitWrite};
+    /// let mut writer = BitWriter::endian(Vec::new(), BigEndian);
+    /// writer.write_signed_out::<4, _>(-5).unwrap();
+    /// writer.write_signed_out::<4, _>(7).unwrap();
+    /// assert_eq!(writer.into_writer(), [0b10110111]);
+    /// ```
+    ///
+    /// ```
+    /// use std::io::Write;
+    /// use bitstream_io::{LittleEndian, BitWriter, BitWrite};
+    /// let mut writer = BitWriter::endian(Vec::new(), LittleEndian);
+    /// writer.write_signed_out::<4, _>(7).unwrap();
+    /// writer.write_signed_out::<4, _>(-5).unwrap();
+    /// assert_eq!(writer.into_writer(), [0b10110111]);
+    /// ```
+    #[inline]
+    fn write_signed_out<const BITS: u32, S>(&mut self, value: S) -> io::Result<()>
+    where
+        S: SignedNumeric,
+    {
+        const {
+            assert!(BITS > 0, "signed writes need at least 1 bit for sign");
+            assert!(BITS <= S::BITS_SIZE, "excessive bits for type written");
+        }
+        E::write_signed_fixed::<_, BITS, S>(self, value)
     }
 
     #[inline]
@@ -672,7 +811,7 @@ impl<W: io::Write, E: Endianness> BitWrite for BitWriter<W, E> {
         if self.byte_aligned() {
             self.writer.write_all(buf)
         } else {
-            buf.iter().try_for_each(|b| self.write(8, *b))
+            buf.iter().try_for_each(|b| self.write_out::<8, _>(*b))
         }
     }
 
@@ -786,12 +925,53 @@ where
         }
     }
 
+    fn write_out<const BITS: u32, U>(&mut self, value: U) -> io::Result<()>
+    where
+        U: Numeric,
+    {
+        const {
+            assert!(BITS <= U::BITS_SIZE, "excessive bits for type written");
+        }
+
+        if (BITS < U::BITS_SIZE) && (value >= (U::ONE << BITS)) {
+            Err(io::Error::new(
+                io::ErrorKind::InvalidInput,
+                "excessive value for bits written",
+            ))
+        } else {
+            self.bits += BITS.into();
+            Ok(())
+        }
+    }
+
     #[inline]
     fn write_signed<S>(&mut self, bits: u32, value: S) -> io::Result<()>
     where
         S: SignedNumeric,
     {
-        E::write_signed(self, bits, value)
+        match bits {
+            0 => Err(io::Error::new(
+                io::ErrorKind::InvalidInput,
+                "signed writes need at least 1 bit for sign",
+            )),
+            bits if bits <= S::BITS_SIZE => E::write_signed(self, bits, value),
+            _ => Err(io::Error::new(
+                io::ErrorKind::InvalidInput,
+                "excessive bits for type written",
+            )),
+        }
+    }
+
+    #[inline]
+    fn write_signed_out<const BITS: u32, S>(&mut self, value: S) -> io::Result<()>
+    where
+        S: SignedNumeric,
+    {
+        const {
+            assert!(BITS > 0, "signed writes need at least 1 bit for sign");
+            assert!(BITS <= S::BITS_SIZE, "excessive bits for type written");
+        }
+        E::write_signed_fixed::<_, BITS, S>(self, value)
     }
 
     #[inline]
@@ -1048,6 +1228,19 @@ where
         Ok(())
     }
 
+    #[inline]
+    fn write_out<const BITS: u32, U>(&mut self, value: U) -> io::Result<()>
+    where
+        U: Numeric,
+    {
+        self.counter.write_out::<BITS, U>(value)?;
+        self.records.push(WriteRecord::Unsigned {
+            bits: BITS,
+            value: value.unsigned_value(),
+        });
+        Ok(())
+    }
+
     #[inline]
     fn write_signed<S>(&mut self, bits: u32, value: S) -> io::Result<()>
     where
@@ -1061,6 +1254,19 @@ where
         Ok(())
     }
 
+    #[inline]
+    fn write_signed_out<const BITS: u32, S>(&mut self, value: S) -> io::Result<()>
+    where
+        S: SignedNumeric,
+    {
+        self.counter.write_signed_out::<BITS, S>(value)?;
+        self.records.push(WriteRecord::Signed {
+            bits: BITS,
+            value: value.signed_value(),
+        });
+        Ok(())
+    }
+
     #[inline]
     fn write_from<V>(&mut self, value: V) -> io::Result<()>
     where
@@ -1122,7 +1328,7 @@ fn write_byte<W>(mut writer: W, byte: u8) -> io::Result<()>
 where
     W: io::Write,
 {
-    writer.write_all(std::slice::from_ref(&byte))
+    writer.write_all(core::slice::from_ref(&byte))
 }
 
 fn write_unaligned<W, E, N>(
@@ -1138,7 +1344,7 @@ where
     if rem.is_empty() {
         Ok(())
     } else {
-        use std::cmp::min;
+        use core::cmp::min;
         let bits_to_transfer = min(8 - rem.len(), acc.len());
         rem.push(bits_to_transfer, acc.pop(bits_to_transfer).to_u8());
         if rem.len() == 8 {
@@ -1160,7 +1366,7 @@ where
         let mut buf = N::buffer();
         let buf_ref: &mut [u8] = buf.as_mut();
         for b in buf_ref[0..to_write].iter_mut() {
-            *b = acc.pop(8).to_u8();
+            *b = acc.pop_fixed::<8>().to_u8();
         }
         writer.write_all(&buf_ref[0..to_write])
     } else {
